'use strict';

module.exports = {
	'#':          require('./#'),
	from:         require('./from'),
	generate:     require('./generate'),
	isPlainArray: require('./is-plain-array'),
	of:           require('./of'),
	toArray:      require('./to-array'),
	validArray:   require('./valid-array')
};
